/**
 * Copyright (c) 2011 ~ 2012 Deepin, Inc.
 *               2011 ~ 2012 jouyouyun
 *
 * Author:      jouyouyun <jouyouwen717@gmail.com>
 * Maintainer:  jouyouyun <jouyouwen717@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 **/

#include "lsearch_speech.h"
#include "dbus_name.h"

#define BUFFER  1024
#define LSEARCH_CONFIG_FILE  "/usr/share/deepin-speech/lsearch_speech.json"

/* dbus glib */
DBusGConnection *conn = NULL;
GError *error = NULL;
DBusGProxy *proxy = NULL;

/* dbus C */
DBusConnection *dbus_conn = NULL;
DBusError dbus_err;
DBusMessage *dbus_msg = NULL;

void exec_speech_rets()
{
    char *pinyin = NULL;

    emit_dbus_signal ("/com/deepin/speech", "com.deepin.speech", 
            "ParseStart", NULL);
    init_config(LSEARCH_CONFIG_FILE);
    pinyin = get_ret_pinyin();

    if ( pinyin == NULL ) {
        g_error ("got pinyin failed in main!\n");
        emit_dbus_signal ("/com/deepin/speech", "com.deepin.speech",
                          "ParseError", NULL);
        return ;
    }

    exec_ret_bin(pinyin);
    finalize_dbus_conn();

    /*emit_dbus_signal ("/com/deepin/speech", "com.deepin.speech", */
            /*"ParseEnd", NULL);*/
    free(pinyin);
    pinyin = NULL;

    return ;
}

void finalize_dbus_conn()
{
    proxy = dbus_g_proxy_new_for_name (conn,
                                     DBUS_CONFIG_NAME,
                                     DBUS_CONFIG_PATH,
                                     DBUS_CONFIG_INFACE);

    error = NULL;
    dbus_g_proxy_call_no_reply(proxy, "finalize_dbus_loop", G_TYPE_INVALID);
    g_object_unref (proxy);
    g_warning("Close config connect\n");

    return ;
}

void lsearch_init()
{
    conn = dbus_g_bus_get(DBUS_BUS_SESSION, &error);

    if ( conn == NULL ) {
        g_warning("connect dbus failed: %s\n", error->message);
        g_error_free(error);
        exit(-1);
    }

    dbus_error_init(&dbus_err);
    dbus_conn = dbus_bus_get(DBUS_BUS_SESSION, &dbus_err);

    if ( dbus_error_is_set(&dbus_err) ) {
        g_printerr("get conn err : %s\n", dbus_err.message);
        dbus_error_free(&dbus_err);
        emit_dbus_signal ("/com/deepin/speech", "com.deepin.speech", 
            "ParseError", NULL);
        exit(-1);
    }

    if ( dbus_conn == NULL ) {
        g_printerr("conn is NULL...\n");
        exit(-1);
    }

    // init_config(LSEARCH_CONFIG_FILE);
    return ;
}

void init_config(const char *path)
{
    int ret = 0;

    if ( path == NULL ) {
        g_warning("init config argument error!\n");
        exit(-1);
    }
    g_warning ("path: %s\n", path);

    proxy = dbus_g_proxy_new_for_name( conn, 
            DBUS_CONFIG_NAME, DBUS_CONFIG_PATH, DBUS_CONFIG_INFACE );

    error = NULL;
    if ( !dbus_g_proxy_call( proxy, "dbus_json_init", &error, 
                G_TYPE_STRING, path, G_TYPE_INVALID, 
                G_TYPE_INT, &ret, G_TYPE_INVALID ) ) {
        if ( (error->domain == DBUS_GERROR) && 
                (error->code == DBUS_GERROR_REMOTE_EXCEPTION) ) {
            g_printerr("Caught remote method exception %s: %s\n", 
                    dbus_g_error_get_name(error), error->message);
        } else {
            g_printerr("Error : %s\n", error->message);
            g_printerr("Error : %s\n", error->message);
        }

        g_error_free(error);
        g_object_unref(proxy);
        emit_dbus_signal ("/com/deepin/speech", "com.deepin.speech", 
            "ParseError", NULL);
        exit(-1);
    }
    g_object_unref(proxy);

    g_warning("json init ret : %d\n", ret);
    if ( ret == -1 ) {
        g_printerr("json init failed!\n");
        finalize_dbus_conn();
        exit(-1);
    }

    return ;
}

char *get_ret_pinyin()
{
    gchar* recog_ret = NULL;

    proxy = dbus_g_proxy_new_for_name( conn,DBUS_SPEECH_NAME, 
            DBUS_SPEECH_PATH, DBUS_SPEECH_INFACE );

    error = NULL;

    if ( !dbus_g_proxy_call( proxy, "dbus_recog", &error,
                             G_TYPE_STRING, RECD_WAV, G_TYPE_INVALID,
                             G_TYPE_STRING, &recog_ret, G_TYPE_INVALID ) ) {
        if ( (error->domain == DBUS_GERROR) &&
                (error->code == DBUS_GERROR_REMOTE_EXCEPTION) ) {
            g_printerr("Caught remote method exception %s: %s\n",
                       dbus_g_error_get_name(error), error->message);
        } else {
            g_warning("in get_pinyin func call recog...\n");
            g_printerr("Error : %s\n", error->message);
        }

        g_error_free(error);
        g_object_unref(proxy);
        //exit(-1);
        emit_dbus_signal ("/com/deepin/speech", "com.deepin.speech", 
            "ParseError", NULL);
        return NULL;
    }

    g_object_unref(proxy);

    if ( recog_ret == NULL ) {
        g_printerr("recog failed...\n");
        emit_dbus_signal ("/com/deepin/speech", "com.deepin.speech", 
            "ParseError", NULL);
        return NULL;
    }

    /* parse rets got pinyin */
    gchar *str_ret = NULL;
    proxy = dbus_g_proxy_new_for_name( conn, DBUS_CONFIG_NAME, 
            DBUS_CONFIG_PATH, DBUS_CONFIG_INFACE );

    error = NULL;

    if ( !dbus_g_proxy_call( proxy, "dbus_parse_ret", &error,
                             G_TYPE_STRING, recog_ret, G_TYPE_INVALID,
                             G_TYPE_STRING, &str_ret, G_TYPE_INVALID ) ) {
        if ( (error->domain == DBUS_GERROR) &&
                (error->code == DBUS_GERROR_REMOTE_EXCEPTION) ) {
            g_printerr("Caught remote method exception %s: %s\n",
                       dbus_g_error_get_name(error), error->message);
        } else {
            g_warning("in get_pinyin func call parse ret...\n");
            g_printerr("Error : %s\n", error->message);
        }

        g_free (recog_ret);
        g_error_free(error);
        g_object_unref(proxy);
        //exit(-1);
        emit_dbus_signal ("/com/deepin/speech", "com.deepin.speech", 
            "ParseError", NULL);
        return NULL;
    }

    g_free (recog_ret);
    g_object_unref(proxy);

    g_warning("------pinyin ret : %s\n", str_ret);    //error

    if ( str_ret == NULL ) {
        g_printerr("parse rets failed...\n");
        emit_dbus_signal ("/com/deepin/speech", "com.deepin.speech", 
            "ParseError", NULL);
        return NULL;
    }

    return str_ret;
}

/*
 * FIXME:修改 exec_ret_bin
 * 使用配置文件中的 cmd 保存 signal，用 type 存储 SpeechText
 * 以此来支持模糊音
 */

void exec_ret_bin(char *pinyin)
{
    char *str_cmd = NULL;
    char *str_type = NULL;

    proxy = dbus_g_proxy_new_for_name( conn, 
            DBUS_CONFIG_NAME, DBUS_CONFIG_PATH, DBUS_CONFIG_INFACE );

    error = NULL;
    dbus_g_proxy_call_no_reply (proxy, "get_ret_via_pinyin", 
        G_TYPE_STRING, pinyin, G_TYPE_INVALID);

    if ( !dbus_g_proxy_call( proxy, "get_cmd", &error, G_TYPE_INVALID, 
                G_TYPE_STRING, &str_cmd,  G_TYPE_INVALID ) ) {
        if ( (error->domain == DBUS_GERROR) && 
                (error->code == DBUS_GERROR_REMOTE_EXCEPTION) ) {
            g_printerr("Caught remote method exception %s: %s\n", 
                    dbus_g_error_get_name(error), error->message);
        } else {
            g_warning("in exec_ret func call get cmd...\n");
            g_printerr("Error : %s\n", error->message);
            g_printerr("Error : %s\n", error->message);
        }

        g_error_free(error);
        g_object_unref(proxy);
        //exit(-1);
        emit_dbus_signal ("/com/deepin/speech", "com.deepin.speech", 
            "ParseError", NULL);
        return ;
    }

    if ( !dbus_g_proxy_call( proxy, "get_type", &error, G_TYPE_INVALID,
            G_TYPE_STRING, &str_type, G_TYPE_INVALID ) ) {
        if ( (error->domain == DBUS_GERROR) && 
                (error->code == DBUS_GERROR_REMOTE_EXCEPTION) ) {
            g_printerr("Caught remote method exception %s: %s\n", 
                    dbus_g_error_get_name(error), error->message);
        } else {
            g_warning("in exec_ret func call get type...\n");
            g_printerr("Error : %s\n", error->message);
            g_printerr("Error : %s\n", error->message);
        }

        g_error_free(error);
        g_object_unref(proxy);
        //exit(-1);
        emit_dbus_signal ("/com/deepin/speech", "com.deepin.speech", 
            "ParseError", NULL);
        return ;
    }
    g_object_unref(proxy);

    g_warning("cmd : %s\n", str_cmd);
    g_warning("type : %s\n", str_type);

    if ( str_cmd[0] == '\0' || str_type[0] == '\0' ) {
        g_printerr("got cmd && type failed...\n");
        emit_dbus_signal ("/com/deepin/speech", "com.deepin.speech", 
            "ParseError", NULL);
        return ;
    }

    if ( strncmp(str_type, "func", 4) != 0 ) {
        emit_dbus_signal (DBUS_LSEARCH_PATH, DBUS_LSEARCH_INFACE, 
                "SpeechText", str_type);
    }
    emit_dbus_signal (DBUS_LSEARCH_PATH, DBUS_LSEARCH_INFACE, 
            str_cmd, NULL);
    emit_dbus_signal ("/com/deepin/speech", "com.deepin.speech", 
            "ParseEnd", NULL);
    
    return ;
}

void emit_dbus_signal(const char *dbus_path, const char *dbus_inface,
                      const char *signame, const char *sigvalue )
{
    DBusConnection *dbus_conn = NULL;
    DBusError dbus_err;
    DBusMessage *msg = NULL;
    DBusMessageIter args;

    dbus_error_init(&dbus_err);
    dbus_conn = dbus_bus_get(DBUS_BUS_SESSION, &dbus_err);

    if ( dbus_error_is_set(&dbus_err) ) {
        g_warning("get conn error : %s\n", dbus_err.message);
        dbus_error_free(&dbus_err);
    }

    if ( dbus_conn == NULL ) {
        g_warning("conn NULL...\n");
        return ;
    }

    msg = dbus_message_new_signal (dbus_path, dbus_inface, signame);

    if ( msg == NULL ) {
        g_warning("msg NULL...\n");
        return ;
    }

    //append args onto signal
    if ( sigvalue ) {
        dbus_message_iter_init_append(msg, &args);

        if ( !dbus_message_iter_append_basic(&args, 
                    DBUS_TYPE_STRING, &sigvalue) ) {
            g_warning("Out Of Memory in append args\n");
            return ;
        }
    }

    //send message
    if ( !dbus_connection_send(dbus_conn, msg, NULL) ) {
        g_warning("Out Of Memory!\n");
        return ;
    }

    dbus_connection_flush(dbus_conn);
    g_warning("%s Signal has been sent...\n", signame);

    dbus_message_unref(msg);
}

#define KEY_LEN 4
#define ONE_KEY 0
#define KEY_PRESSED 1
#define KEY_RELEASED 2

void dbus_send_keys_event(unsigned long xid, const char *buf_keys)
{
    int len = 0;
    int keys[KEY_LEN];
    int *tmp_key = NULL;
    Display *disp = NULL;
    
    if ( buf_keys == NULL ) {
        g_warning("argument NULL in in dbus_exec_via_keys\n");
        return ;
    }
    
    memset(keys, 0, KEY_LEN * sizeof(int));
    len = strtok_num ((char*)buf_keys, keys);
    tmp_key = keys;
    
    setlocale(LC_ALL, "");

    if ( !(disp = XOpenDisplay(NULL)) ) {
        g_warning("Cannot open display...\n");
        return ;
    }
    g_warning("create display ok!\n");
    
    if ( xid != 0 ) {
        /* 设置指定窗口为焦点 */
        XSetInputFocus(disp, (Window)xid, RevertToParent, CurrentTime);
        XFlush(disp);
    }
    while ( XPending(disp) ) {}

    g_warning ("key len: %d\n", len);
    if ( len == 1 ) {
        set_key_state (disp, *tmp_key, ONE_KEY);
        XCloseDisplay(disp);
        return;
    }
    g_warning("set keys pressed...\n");
    for (; *tmp_key != 0; tmp_key++ ) {
        set_key_state (disp, *tmp_key, KEY_PRESSED);
    }

    /*g_warning("set keys release...\n");*/
    tmp_key = keys;
    for (; *tmp_key != 0; tmp_key++ ) {
        set_key_state (disp, *tmp_key, KEY_RELEASED);
    }

    XCloseDisplay(disp);
}

/*
 * 分割按键字符串
 * 得到按键的整型值
 */
int strtok_num(char *exec_buf, int *nums)
{
    int len = 0;
    char *p = NULL;
    char *tmp = NULL;
    
    p = strtok(exec_buf, " ");
    *nums = strtol(p, &tmp, 16);
    printf("key num:%d\t0x%x\n", *nums, *nums);
    len++;

    nums++;
    //keys_len++;
    while ( (p = strtok(NULL, " ")) ) {
        *nums = strtol(p, &tmp, 16);
        printf("%d\n", *nums);
        nums++;
        len++;
    }
    *nums = 0;

    return len;
}

/*
 * 设置按键为 Pressed/Released 状态
 */
void set_key_state( Display *disp, int keycode, int state )
{
    if ( state == ONE_KEY ) {
        XTestFakeKeyEvent(disp, XKeysymToKeycode(disp, keycode), 
                True, CurrentTime);
        XTestFakeKeyEvent(disp, XKeysymToKeycode(disp, keycode), 
                False, CurrentTime);
    } else if ( state == KEY_PRESSED ) {
        // 设置按键为 Pressed 状态
        g_warning ("Press key!\n");
        XTestFakeKeyEvent(disp, XKeysymToKeycode (disp, keycode), 
                True, CurrentTime);
    } else if ( state == KEY_RELEASED ){
        // 设置按键为 Released 状态
        g_warning ("Released key!\n");
        XTestFakeKeyEvent(disp, XKeysymToKeycode(disp, keycode), 
                False, CurrentTime);
    }

    XFlush(disp);
    g_warning ("Exec keys events...\n");
}
