/*
 * gcc -Wall -g test_dbus_c.c -o Test `pkg-config --cflags --libs dbus-1`
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dbus/dbus.h>
#include "dbus_name.h"

#define sys_err(args ...)   { fprintf(stderr, args); }
#define sys_says(args ...)   { fprintf(stdout, args); }

int main()
{
    DBusConnection *conn = NULL;
    DBusMessageIter args;
    DBusMessage *msg = NULL;
    DBusError err;
    DBusPendingCall *pending = NULL;
    dbus_uint32_t xid = 0;

    dbus_error_init(&err);
    conn = dbus_bus_get(DBUS_BUS_SESSION, &err);
    if ( dbus_error_is_set(&err) ) {
        sys_err("Connection Error: %s\n", err.message);
        dbus_error_free(&err);
    }
    if ( conn == NULL ) {
        return -1;
    }

    msg = dbus_message_new_method_call(DBUS_WNCK_NAME, 
            DBUS_WNCK_PATH, 
            DBUS_WNCK_INFACE, 
            "get_cur_xid");
    if ( msg == NULL ) {
        sys_err("Message NULL!!\n");
        return -1;
    }

    //append arguments
    //char *param = NULL;
    //dbus_message_iter_init_append(msg, &args);
    //if ( !dbus_message_iter_append_basic(&args, DBUS__TYPE_STRING, 
    //            &param)) {
    //    sys_err("Out of Memory in append arguments!\n");
    //    return -1;
    //}

    //send message and get a handle for a reply
    if ( !dbus_connection_send_with_reply(conn, msg, &pending, -1) ) {
        sys_err("Out of Memory in send message!\n");
        return -1;
    }

    if ( pending == NULL ) {
        sys_err("pending Call NULL!\n");
        return -1;
    }
    dbus_connection_flush(conn);
    sys_says("Request Send\n");

    //free message
    dbus_message_unref(msg);

    //block until we recieve a reply
    dbus_pending_call_block(pending);

    //got reply message
    msg = dbus_pending_call_steal_reply(pending);
    if ( msg == NULL ) {
        sys_err("Reply NULL!\n");
        return -1;
    }
    dbus_pending_call_unref(pending);

    //read the parameters
    if ( !dbus_message_iter_init(msg, &args) ) {
        sys_err("Message has no arguments!\n");
    } else if ( dbus_message_iter_get_arg_type(&args) != DBUS_TYPE_UINT32 ) {
        sys_err("Arguments is not int32!\n");
    } else {
        dbus_message_iter_get_basic(&args, &xid);
    }

    sys_says("xid : %lu\n", xid);
    dbus_message_unref(msg);

    return 0;
}
