/*
 * Compile:
 * gcc -Wall test.c -o Test -g `pkg-config --cflags --libs glib-2.0 dbus-glib-1`
 */
#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <dbus/dbus-glib.h>
#include "dbus_name.h"

#define sys_err(args ...)   { fprintf(stderr, args); }
#define sys_says(args ...)   { fprintf(stdout, args); }

int main()
{
    /* dbus glib */
    DBusGConnection *glib_conn = NULL;
    DBusGProxy *glib_proxy = NULL;
    GError *glib_err = NULL;
    unsigned long app_xid = 0;

    glib_conn = dbus_g_bus_get(DBUS_BUS_SESSION, &glib_err);
    if ( glib_conn == NULL ) {
        sys_err("glib conn got conn failed: %s\n", glib_err->message);
        g_error_free(glib_err);
        exit(-1);
    }

    glib_proxy = dbus_g_proxy_new_for_name(glib_conn, 
            DBUS_WNCK_NAME, DBUS_WNCK_PATH, DBUS_WNCK_INFACE);

    sys_says("get focus name ...\n");
    glib_err = NULL;
    if ( !dbus_g_proxy_call(glib_proxy, "get_cur_xid", &glib_err, 
                G_TYPE_INVALID, 
                G_TYPE_UINT, &app_xid, G_TYPE_INVALID) ) {
        if ( (glib_err->domain == DBUS_GERROR) && 
                (glib_err->code == DBUS_GERROR_REMOTE_EXCEPTION) ) {
            sys_err("Caught remote method exception %s: %s\n", 
                    dbus_g_error_get_name(glib_err), glib_err->message );
        } else {
            sys_err("in get_focus_app_name got xid...\n");
            sys_err("Error : %s\n", glib_err->message);
        }

        sys_says("conn wnck error...\n");
        g_error_free(glib_err);
        g_object_unref(glib_proxy);
        return -1;
    }

    sys_says("--------app xid : %lu\n", app_xid);
    if ( app_xid == 0 ) {
        sys_err("got app xid failed...\n");
        return -1;
    }

    glib_err = NULL;
    char *app_class = NULL;
    if ( !(dbus_g_proxy_call(glib_proxy, "get_window_class", &glib_err, 
                    G_TYPE_ULONG, app_xid, G_TYPE_INVALID, 
                    G_TYPE_STRING, &app_class, G_TYPE_INVALID)) ) {
        if ( (glib_err->domain == DBUS_GERROR) && 
                (glib_err->code == DBUS_GERROR_REMOTE_EXCEPTION) ) {
            sys_err("Caught remote method exception %s: %s\n", 
                    dbus_g_error_get_name(glib_err), glib_err->message );
        } else {
            sys_err("in get_focus_app_name got win class...\n");
            sys_err("Error : %s\n", glib_err->message);
        }

        g_error_free(glib_err);
        g_object_unref(glib_proxy);
        return -1;
    }

    sys_says("--------app class : %s\n", app_class);
    if ( app_class == NULL ) {
        sys_err("got app class is NULL\n");
        return -1;
    }

    return 0;
}
