/**
 * Copyright (c) 2011 ~ 2012 Deepin, Inc.
 *               2011 ~ 2012 jouyouyun
 *
 * Author:      jouyouyun <jouyouwen717@gmail.com>
 * Maintainer:  jouyouyun <jouyouwen717@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 **/

#include "parse_json.h"
#include <iostream>
#include <string>
#include <fstream>
#include <json/json.h>

using namespace std;

struct cmd_info *cmds_json = NULL;
GHashTable *cmd_table = NULL;

void print_key_value(gpointer key, gpointer value, gpointer user_data)
{
    g_print("%s ---> %s\n", (char*)key, (char*)value);
}

/*
 * 建立 hash 表存储拼音与 cmd_info 下标
 * 通过 hash 表找到数组下标，通过数组下标找到需要执行的命令
 */
 
struct cmd_info *ParseJsonFromFile(const char *file)
{
    Json::Reader reader;
    Json::Value index;
    std::string str;
    int i;
    int flag = 0;
    int arraylen;
    char tmp[BUF_LEN];
    struct cmd_info *cmd_cur = NULL;
    struct cmd_info *cmd_tmp = NULL;
    
    if ( cmd_table == NULL || file == NULL ) {
        sys_says("parse json args error\n");
        return NULL;
    }

    sys_says("ParseJsonFromFile : open %s\n", file);
    std::ifstream is;
    is.open(file, std::ios::binary);

    sys_says("ParseJsonFromFile : open successful\n");
    if ( reader.parse(is, index) ) {
        sys_says("ParseJsonFromFile : parse start...\n");
        arraylen = index["index"].size();
        std::cout<<"arraylen : "<<arraylen<<endl;

        for ( i = 0; i < arraylen; i++ ) {
            if ( i == 0 ) {
                cmd_cur = (struct cmd_info *)calloc(i + 1, 
                                            sizeof(struct cmd_info));
                if ( cmd_cur == NULL ) {
                    sys_says("cmd cur alloc error\n");
                    break;
                }
            } else {
                cmd_tmp = (struct cmd_info *)calloc(i + 1, 
                                            sizeof(struct cmd_info));
                if ( cmd_tmp == NULL ) {
                    sys_says("cmd tmp alloc error\n");
                    break;
                }
                memcpy(cmd_tmp, cmd_cur, i * sizeof(struct cmd_info));
                free(cmd_cur);
                cmd_cur = NULL;
                cmd_cur = cmd_tmp;
                cmd_tmp = NULL;
            }
            str = index["index"][i]["pinyin"].asString();
            memset(tmp, 0, BUF_LEN);
            strcpy(tmp, str.c_str());
            //printf("tmp : %s\tlen : %d\n", tmp, strlen(tmp));
            
            //hash insert
            gchar *gstrs = g_strdup_printf("%s", tmp);
            gchar *str_num = g_strdup_printf("%i", i);
            sys_says("insert keys : %s\tvalue : %s\n", gstrs, str_num);
            g_hash_table_insert(cmd_table, (void*)gstrs, (void*)str_num);
            g_hash_table_foreach(cmd_table, print_key_value, NULL);
            //g_free(gstrs);
            //g_free(str_num);

            str = index["index"][i]["cmd"].asString();
            memset(tmp, 0, BUF_LEN);
            strcpy(tmp, str.c_str());
            memcpy((cmd_cur + i)->cmd, tmp, strlen(tmp));

            str = index["index"][i]["type"].asString();
            memset(tmp, 0, BUF_LEN);
            strcpy(tmp, str.c_str());
            memcpy((cmd_cur + i)->type, tmp, strlen(tmp));
            sys_says("-----****cmd : %s\ttype : %s\n\n", (cmd_cur + i)->cmd, 
                        (cmd_cur + i)->type);
        }
    } else {
        sys_says("read json file error!!!\n");
    }
    is.close();
    if ( cmd_cur == NULL ) {
        sys_says("parse json cmd_cur NULL\n");
    }

    return cmd_cur;
}

int dbus_json_init(const char *path)
{
    cmd_table = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
    sys_says("json path : %s\n", path);
    cmds_json = ParseJsonFromFile(path);
    if ( cmds_json == NULL || cmd_table == NULL ) {
        sys_err("parse json init file error...\n");
        return -1;
    }
    
    g_hash_table_foreach(cmd_table, print_key_value, NULL);
    
    return 0;
}

static char cmd_ret[CMD_LEN];
static char type_ret[TYPE_LEN];

char *get_cmd()
{
    return cmd_ret;
}

char *get_type()
{
    return type_ret;
}

void get_ret_via_pinyin(const char *pinyin)
{

    if ( cmds_json == NULL || cmd_table == NULL || pinyin == NULL ) {
        sys_err("parse json file error...\n");
        return ;
    }

    int num = 0;
    char *str_num = NULL;
    struct cmd_info *rets = (struct cmd_info *)calloc(1, sizeof(struct cmd_info));
    if ( rets == NULL ) {
        sys_err("got memory failed in get ret via pinyin...\n");
        return ;
    }
    
    sys_says("search keys : %s\n", pinyin);
    str_num = (char*)g_hash_table_lookup(cmd_table, pinyin);
    if ( str_num == NULL ) {
        return ;
    }

    sys_says("str value : %s\n", str_num);
    num = atoi(str_num);
    sys_says("num : %d\tstr num : %s\n", num, str_num);
    sys_says("cmd : %s\ttype : %s\n", (cmds_json + num)->cmd, 
                        (cmds_json + num)->type);
    memcpy(rets->cmd, (cmds_json + num)->cmd, strlen((cmds_json + num)->cmd));
    memcpy(rets->type, (cmds_json + num)->type, strlen((cmds_json + num)->type));

    memset(cmd_ret, 0, CMD_LEN);
    memset(type_ret, 0, TYPE_LEN);
    memcpy(cmd_ret, rets->cmd, strlen(rets->cmd));
    memcpy(type_ret, rets->type, strlen(rets->type));
    free(rets);
    rets = NULL;
    
    return ;
}

struct cmd_info *dbus_get_cmd(const char *pinyin)
{
    int num = 0;
    char *str_num = NULL;
    struct cmd_info *rets = (struct cmd_info *)calloc(1, sizeof(struct cmd_info));
    
    if ( cmds_json == NULL || cmd_table == NULL || rets == NULL ) {
        sys_err("parse json file error...\n");
        return NULL;
    }
    
    sys_says("search keys : %s\n", pinyin);
    str_num = (char*)g_hash_table_lookup(cmd_table, pinyin);
    sys_says("str value : %s\n", str_num);
    num = atoi(str_num);
    sys_says("num : %d\tstr num : %s\n", num, str_num);
    sys_says("cmd : %s\ttype : %s\n", (cmds_json + num)->cmd, 
                        (cmds_json + num)->type);
    memcpy(rets->cmd, (cmds_json + num)->cmd, strlen((cmds_json + num)->cmd));
    memcpy(rets->type, (cmds_json + num)->type, strlen((cmds_json + num)->type));
    
    return rets;
}

void hash_table_destroy()
{
    if ( cmds_json != NULL ) {
        free(cmds_json);
        cmds_json = NULL;
    }
    
    if ( cmd_table != NULL ) {
        g_hash_table_destroy(cmd_table);
        cmd_table = NULL;
    }
}
