/**
 * Copyright (c) 2011 ~ 2012 Deepin, Inc.
 *               2011 ~ 2012 jouyouyun
 *
 * Author:      jouyouyun <jouyouwen717@gmail.com>
 * Maintainer:  jouyouyun <jouyouwen717@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 **/

#include "parse_ret.h"

char ret_pinyin[BUF_LEN];

void finalize_dbus_conn()
{
    DBusGConnection *connection;
    GError *error;
    DBusGProxy *proxy;

    error = NULL;
    connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
    if (connection == NULL) {
        g_printerr ("Failed to open connection to bus: %s\n",
                  error->message);
        g_error_free (error);
        exit (1);
    }
    proxy = dbus_g_proxy_new_for_name (connection,
                                     DBUS_PINYIN_NAME,
                                     DBUS_PINYIN_PATH,
                                     DBUS_PINYIN_INFACE);

    error = NULL;
    dbus_g_proxy_call_no_reply(proxy, "finalize_dbus_loop", G_TYPE_INVALID);
    g_object_unref (proxy);

    return ;
}

char *dbus_parse_ret(char *str)
{
    int len = 0;
    char *py = NULL;
    char *tmp = NULL;

    len = strlen (str);
    fprintf(stderr, "buf : %s\nret : %d\n", str, len);
    tmp = str + (len - 3);
    fprintf(stderr, "tmp : %s\n", tmp);
    clear_punctuation(tmp);
    fprintf(stderr, "cmd_buf : %s\n", str);
    
    tmp = NULL;
    tmp = (char*)calloc(1, len + 1);
    if ( tmp == NULL ) {
        return NULL;
    }
    memcpy(tmp, str, len);
    sys_says("parse ret : %s\n", tmp);
    
    memset(ret_pinyin, 0, BUF_LEN);
    py = get_pinyin(tmp);
    free(tmp);
    tmp = NULL;

    memcpy(ret_pinyin, py, strlen(py));
    sys_says("py : %s\n", py);
    finalize_dbus_conn();
    free(py);
    py = NULL;
    
    return ret_pinyin;
}

/*
 * 去掉多余的符号
 */
void clear_punctuation(char *punc)
{
    if ( strncmp(punc, "。", 3) == 0 ) {
        memset(punc, 0, 3);
    } else if ( strncmp(punc, "？", 3) == 0 ) {
        memset(punc, 0, 3);
    } else if ( strncmp(punc, "，", 3) == 0 ) {
        memset(punc, 0, 3);
    } else if ( strncmp(punc, "！", 3) == 0 ) {
        memset(punc, 0, 3);
    }
}

char *get_pinyin(const char *pinyin)
{
    DBusGConnection *connection;
    GError *error;
    DBusGProxy *proxy;
    char *ret_num = NULL;

    error = NULL;
    connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
    if (connection == NULL) {
        g_printerr ("Failed to open connection to bus: %s\n",
                  error->message);
        g_error_free (error);
        exit (1);
    }
    proxy = dbus_g_proxy_new_for_name (connection,
                                     DBUS_PINYIN_NAME,
                                     DBUS_PINYIN_PATH,
                                     DBUS_PINYIN_INFACE);

    error = NULL;
    if (!dbus_g_proxy_call (proxy, "dbus_get_pinyin", &error, 
                        G_TYPE_STRING, pinyin, 
                        G_TYPE_INVALID,
                        G_TYPE_STRING, &ret_num, G_TYPE_INVALID)) {
        if ((error->domain == DBUS_GERROR) && 
                (error->code == DBUS_GERROR_REMOTE_EXCEPTION)) {
            g_printerr ("Caught remote method exception %s: %s",
                        dbus_g_error_get_name (error), error->message);
        } else {
            g_printerr ("Error: %s\n", error->message);
            g_printerr ("Error: %s\n", error->message);
        }
        
        g_error_free (error);
        //exit (1);
        return NULL;
    }
    
    if ( ret_num == NULL ) {
        g_printerr("return false\n");
        return NULL;
    } else {
        g_printerr("ret : %s\n", ret_num);
    }
    g_object_unref (proxy);

    return ret_num;
}
