/**
 * Copyright (c) 2011 ~ 2012 Deepin, Inc.
 *               2011 ~ 2012 jouyouyun
 *
 * Author:      jouyouyun <jouyouwen717@gmail.com>
 * Maintainer:  jouyouyun <jouyouwen717@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 **/

#include "dbus_speech.h"

//internal functions
static gboolean _retry_registration (gpointer user_data);
static void _on_bus_acquired (GDBusConnection * connection, const gchar * name, gpointer user_data);
static void _on_name_acquired (GDBusConnection * connection, const gchar * name, gpointer user_data);
static void _on_name_lost (GDBusConnection * connection, const gchar * name, gpointer user_data);
static void _bus_method_call (GDBusConnection * connection, const gchar * sender,
                             const gchar * object_path, const gchar * interface,
                             const gchar * method, GVariant * params,
                             GDBusMethodInvocation * invocation, gpointer user_data);
static gboolean do_exit(gpointer user_data);

GMainLoop *loop = NULL;

static guint lock_service_owner_id;
static guint lock_service_reg_id;        //used for unregister an object path
static guint retry_reg_timeout_id;   //timer used for retrying dbus name registration.
GDBusConnection* lock_connection = NULL;

static GDBusNodeInfo *      node_info = NULL;
static GDBusInterfaceInfo *interface_info = NULL;
static GDBusInterfaceVTable interface_table = {
    method_call:   _bus_method_call,
    get_property:   NULL, // No properties 
    set_property:   NULL  // No properties 
};

const char *_lock_dbus_iface_xml = 
"<?xml version=\"1.0\"?>\n"
"<node>\n"
"	<interface name=\""DBUS_SPEECH_INFACE"\">\n"
"		<method name=\"dbus_synth\">\n"
"		<arg name=\"str\" type=\"s\" direction=\"in\">\n"
"		</arg>\n"
"		<arg name=\"dest_file\" type=\"s\" direction=\"in\">\n"
"		</arg>\n"
"		<arg name=\"ret\" type=\"i\" direction=\"out\">\n"
"		</arg>\n"
"		</method>\n"
"		<method name=\"dbus_recog\">\n"
"		<arg name=\"pcm_file\" type=\"s\" direction=\"in\">\n"
"		</arg>\n"
"		<arg name=\"ret\" type=\"s\" direction=\"out\">\n"
"		</arg>\n"
"		</method>\n"
"		<method name=\"dbus_gst_play\">\n"
"		<arg name=\"file\" type=\"s\" direction=\"in\">\n"
"		</arg>\n"
"		<arg name=\"type\" type=\"s\" direction=\"in\">\n"
"		</arg>\n"
"		<arg name=\"repeat\" type=\"i\" direction=\"in\">\n"
"		</arg>\n"
"		</method>\n"
"		<method name=\"dbus_gst_stop\">\n"
"		</method>\n"
"		<method name=\"dbus_read_str\">\n"
"		<arg name=\"str\" type=\"s\" direction=\"in\">\n"
"		</arg>\n"
"		<arg name=\"dest_wav\" type=\"s\" direction=\"in\">\n"
"		</arg>\n"
"		</method>\n"
"		<method name=\"dbus_read_xsel\">\n"
"		<arg name=\"dest_wav\" type=\"s\" direction=\"in\">\n"
"		</arg>\n"
"		</method>\n"
"		<method name=\"finalize_dbus_loop\">\n"
"		</method>\n"
"		<method name=\"record_pulse\">\n"
"		</method>\n"
"		<signal name=\"RecordStart\">\n"
"		</signal>\n"
"		<signal name=\"RecordEnd\">\n"
"		</signal>\n"
"		<signal name=\"ParseStart\">\n"
"		</signal>\n"
"		<signal name=\"ParseEnd\">\n"
"		</signal>\n"
"		<signal name=\"CurrentVolume\">\n"
"		<arg name=\"volume\" type=\"d\" direction=\"out\">\n"
"		</arg>\n"
"		</signal>\n"
"		<signal name=\"ParseError\">\n"
"		</signal>\n"
"	</interface>\n"
"</node>\n"
;

void dbus_speech_inface()
{
	loop = g_main_loop_new(NULL, FALSE);
	GError *error = NULL;
	
	node_info = g_dbus_node_info_new_for_xml (_lock_dbus_iface_xml, &error);
	if ( error != NULL ) {
		g_critical("Unable to parse interface xml: %s\n", error->message);
		g_error_free(error);
	}

	interface_info = g_dbus_node_info_lookup_interface(node_info, 
			DBUS_SPEECH_INFACE );
	if ( interface_info == NULL ) {
		g_critical("Unable to find interface '"DBUS_SPEECH_INFACE"'");
	}

	lock_service_owner_id = 0;
	lock_service_reg_id = 0;
	retry_reg_timeout_id = 0;
	_retry_registration(NULL);
	
//	g_timeout_add_seconds( 20, do_exit, NULL );
	g_main_loop_run(loop);
	g_main_loop_unref(loop);

	return ;
}

void finalize_dbus_loop()
{
	g_main_loop_quit(loop);
}

static gboolean
do_exit(gpointer user_data)
{
	g_main_loop_quit(loop);

	return FALSE;
}

static gboolean
_retry_registration ( gpointer user_data )
{
	lock_service_owner_id = g_bus_own_name( G_BUS_TYPE_SESSION, 
				DBUS_SPEECH_NAME, 
				G_BUS_NAME_OWNER_FLAGS_NONE, 
				lock_service_reg_id ? NULL : _on_bus_acquired,
				_on_name_acquired,
				_on_name_lost, 
				NULL, 
				NULL);
	return 0;
}

static void
_on_bus_acquired ( GDBusConnection *connection, 
		const gchar *name, 
		gpointer user_data )
{
	g_debug(" on bus acquired ...\n");

	lock_connection = connection;

	//register object
	GError *error = NULL;
	lock_service_reg_id = g_dbus_connection_register_object( connection, 
			DBUS_SPEECH_PATH, 
			interface_info, 
			&interface_table, 
			user_data, 
			NULL, 
			&error );

	if ( error != NULL ) {
		g_critical ( "Unable to register object to the dbus: %s\n", 
				error->message );
		g_error_free(error);
		g_bus_unown_name(lock_service_owner_id);
		lock_service_owner_id = 0;
		retry_reg_timeout_id = g_timeout_add_seconds(1, 
				_retry_registration, NULL );
		return;
	}

	return;
}

static void
_on_name_acquired ( GDBusConnection *connection, 
		const gchar* name, 
		gpointer user_data )
{
	g_debug ( "Dbus name acquired ... \n" );
}

static void
_on_name_lost ( GDBusConnection *connection, 
		const gchar *name, 
		gpointer user_data )
{
	if ( connection == NULL ) {
		g_critical ( "Unable to get a connection to DBus...\n" );
	} else {
		g_critical ( "Unable to claim the name %s\n", DBUS_SPEECH_NAME );
	}

	lock_service_owner_id = 0;
}

static void
_bus_method_call (GDBusConnection * connection,
                 const gchar * sender, const gchar * object_path, const gchar * interface,
                 const gchar * method, GVariant * params,
                 GDBusMethodInvocation * invocation, gpointer user_data)
{
    g_debug ("bus_method_call");

    GVariant * retval = NULL;
    GError * error = NULL;

    if (g_strcmp0 (method, "dbus_synth") == 0) {
        const gchar *str1 = NULL;
        const gchar *str2 = NULL;
		
		g_variant_get (params, "(ss)", &str1, &str2);
		retval = g_variant_new("(i)", dbus_synth(str1, str2));
    } else if (g_strcmp0 (method, "dbus_recog") == 0) {
        const gchar *str1 = NULL;
        //gchar *recog_ret = NULL;
		
		g_variant_get (params, "(s)", &str1);
		retval = g_variant_new("(s)", dbus_recog(str1));
    }  else if (g_strcmp0 (method, "dbus_gst_play") == 0) {
        const gchar *file = NULL;
        const gchar *type = NULL;
        int repeat = 0;
		
		g_variant_get (params, "(ssi)", &file, &type, &repeat);
		dbus_gst_play(file, type, repeat);
    } else if ( g_strcmp0 (method, "dbus_gst_stop") == 0) {
		dbus_gst_stop ();
	} else if ( g_strcmp0 (method, "dbus_read_xsel") == 0) {
		const gchar *dest_wav = NULL;
		
		g_variant_get (params, "(s)", &dest_wav);
		dbus_read_xsel (dest_wav);
    } else if ( g_strcmp0 (method, "dbus_read_str") == 0) {
		const gchar *str = NULL;
		const gchar *dest_wav = NULL;
		
		g_variant_get (params, "(ss)", &str, &dest_wav);
		dbus_read_str (str, dest_wav);
    } else if (g_strcmp0 (method, "finalize_dbus_loop") == 0) {
		finalize_dbus_loop();
	} else if (g_strcmp0 (method, "record_pulse") == 0) {
		record_pulse();
	} else {
        g_warning ("Calling method '%s' on lock and it's unknown", method);
    }

    if (error != NULL) {
        g_dbus_method_invocation_return_dbus_error (invocation,
                "com.deepin.dde.lock.Error",
                error->message);
        g_error_free (error);

    } else {
        g_dbus_method_invocation_return_value (invocation, retval);
    }
}

void dbus_sig_emit(const char *signame, const double params, int flag)
{
	GError *error = NULL;

	if ( flag == 1 ) {
		g_dbus_connection_emit_signal (lock_connection,
									DBUS_SPEECH_NAME,
									DBUS_SPEECH_PATH,
									DBUS_SPEECH_INFACE,
									signame,
									g_variant_new ("(d)", &params),
									&error);
	} else {
		g_dbus_connection_emit_signal (lock_connection,
									DBUS_SPEECH_NAME,
									DBUS_SPEECH_PATH,
									DBUS_SPEECH_INFACE,
									signame,
									NULL,
									&error);
	}
	g_assert_no_error (error);
	sys_says("------------emit %s---------\n", signame);
}

void test_signal_handler(GDBusConnection * connection, 
				const gchar * sender_name,
				const gchar * object_path,
				const gchar * interface_name,
				const gchar * signal_name,
				GVariant * parameters, gpointer user_data)
{
	fprintf( stderr, "<<<<<<<<< test signal handler...\n");
	fprintf( stderr, ">>>>>>>>> recv a signal ...\n");
}

void listen_dbus_signal(const char *name)
{
	GError *error = NULL;
	
	if ( lock_connection != NULL ) {
		sys_says("listen dbus signal");
		
		g_dbus_connection_signal_subscribe(lock_connection, DBUS_SPEECH_NAME,
            DBUS_SPEECH_NAME,
            name,
            DBUS_SPEECH_PATH,
            NULL,
            G_DBUS_SIGNAL_FLAGS_NONE,
            test_signal_handler,
            NULL, NULL);
	}
}
