/**
 * Copyright (c) 2011 ~ 2012 Deepin, Inc.
 *               2011 ~ 2012 jouyouyun
 *
 * Author:      jouyouyun <jouyouwen717@gmail.com>
 * Maintainer:  jouyouyun <jouyouwen717@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 **/

#include "dbus_speech.h"

#include <glib.h>
#include <gst/gst.h>

static void bus_callback(GstBus *bus, GstMessage *message, gpointer data);

GstElement *pipeline = NULL;
static int repeat_flag = 0;

void
dbus_gst_play (const char *file, const char *type, int repeat)
{
    gchar *launch_cmd = NULL;
    GError *error = NULL;
    GstBus *bus = NULL;

    if ( file == NULL ) {
        g_warning ("pcm file path error!\n");
        return ;
    }

    dbus_gst_stop();
    g_print ("audio type: %s\n", type);
    if ( g_strcmp0("pcm", type) == 0 ) {
        launch_cmd = g_strdup_printf ("filesrc location=\"%s\" \
                ! audio/x-raw-int, width=16, depth=16, channels=1, rate=16000, \
                signed=true ! alsasink", file);
    } else {
        launch_cmd = g_strdup_printf ("filesrc location=\"%s\" ! decodebin2 ! \
            alsasink", file);
    }
    if ( launch_cmd == NULL ) {
        g_warning ("launch cmd is NULL!\n");
        return ;
    }

    if ( repeat == 1 ) {
        repeat_flag = 1;
    }
    
    g_print ("gst launch : %s\n", launch_cmd);
    gst_init (NULL, NULL);
    pipeline = gst_parse_launch (launch_cmd, &error);
    if ( !pipeline ) {
        g_warning ("create pipe failed\n");
        if (error) {
            g_warning ("Error: %s\n", error->message);
            g_error_free (error);
            error = NULL;
        }
        return ;
    }

    bus = gst_pipeline_get_bus (GST_PIPELINE (pipeline));
    gst_bus_add_watch (bus, (GstBusFunc)bus_callback, NULL);
    gst_object_unref (GST_OBJECT (bus));

    gst_element_set_state (pipeline, GST_STATE_PLAYING);
}

static void bus_callback(GstBus *bus, GstMessage *message, gpointer data)
{
    gchar *message_str;
    GError *error;
    
    /* Report errors to the console */
    switch (GST_MESSAGE_TYPE (message)) {
        case GST_MESSAGE_EOS:
            g_print ("End of stream\n");
            if ( repeat_flag == 1) {
                gst_element_seek (pipeline,1.0, GST_FORMAT_TIME, 
                        GST_SEEK_FLAG_FLUSH, GST_SEEK_TYPE_SET, 
                        1, //2 seconds (in nanoseconds)
                        GST_SEEK_TYPE_NONE, GST_CLOCK_TIME_NONE);
            } else {
                dbus_gst_stop ();
            }
            break;
        case GST_MESSAGE_ERROR:
            gst_message_parse_error(message, &error, &message_str);
            g_warning("GST error: %s\n", message_str);
            g_free(error);
            g_free(message_str);
            break;
        default:
            break;
    }
}

void
dbus_gst_stop ()
{
    repeat_flag = 0;
    if ( pipeline ) {
        g_print ("Finalize Play...\n");
        gst_element_set_state (pipeline, GST_STATE_NULL);
        gst_object_unref(GST_OBJECT(pipeline));
        g_print ("Finalize Finish...\n");
    }
}

void dbus_read_str (const char* str, const char* dest_wav)
{
    int ret = 0;

    if ( str == NULL || dest_wav == NULL ) {
        g_warning ("params is null");
        return ;
    }

    ret = dbus_synth (str, dest_wav);
    if ( ret == -1 ) {
        g_warning ("synth str failed in read string!\n");
        return ;
    }

    dbus_gst_play (dest_wav, NULL, 0);
}

void dbus_read_xsel(const char *dest_wav)
{
    gchar* result = NULL;
    const char cmd[] = "xsel";

    if ( !g_spawn_command_line_sync(cmd, &result, NULL, NULL, NULL) ) {
        g_warning ("get xsel string failed!\n");
        return ;
    }

    dbus_read_str (result, dest_wav);
    g_free (result);
}
