/**
 * Copyright (c) 2011 ~ 2012 Deepin, Inc.
 *               2011 ~ 2012 jouyouyun
 *
 * Author:      jouyouyun <jouyouwen717@gmail.com>
 * Maintainer:  jouyouyun <jouyouwen717@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 **/

#include "record.h"
#include <dbus/dbus.h>
//#include <alsa/asoundlib.h>

#define WAIT_TIME_SEC  1
#define WAIT_TIME_USEC  500

static int cnt = 0;
int quit_flag = 0;

pa_stream *s = NULL;
double first_db = 0.0;
double cur_db = 0.0;

void record_end()
{
    while (1) {
        if ( cur_db > (first_db + 0.0005) ) {
            first_db = cur_db;
            continue;
        }

        sys_says("cur db : %lf\t", cur_db);
        sys_says("rang db : %lf\n", (first_db + 0.0005) );

        if ( cnt >= 3 ) {
            quit_flag = 1;
            cnt = 0;
            break;
        }

        if ( cnt == 0 ) {
            sleep(WAIT_TIME_SEC);
        } else {
            sleep(WAIT_TIME_SEC);
            //usleep( (WAIT_TIME_USEC) * 1000 );
        }

        cnt++;
    }

    g_thread_exit(NULL);
}

void parse_record()
{
    static const pa_sample_spec ss = {
        .format = PA_SAMPLE_S16LE,
        .rate = 16000,
        .channels = 1
    };
    pa_simple *ps = NULL;
    int ret = 1;
    int error;
    FILE *fp = NULL;
    GThread *quit_thrd = NULL;

    /* Create the recording stream */
    if (!(ps = pa_simple_new(NULL, "deepin", PA_STREAM_RECORD, NULL,
                             "record", &ss, NULL, NULL, &error))) {
        fprintf(stderr, __FILE__": pa_simple_new() failed: %s\n",
                pa_strerror(error));
        goto finish;
    }

    fp = fopen("/tmp/recd_tmp.wav", "w");

    if ( fp == NULL ) {
        printf( "fopen file failed...\n" );
        g_thread_exit(NULL);
    }

    //end record
    cnt = 0;
    quit_thrd = g_thread_new("exit record", (GThreadFunc)record_end, NULL);

    if ( quit_thrd == 0 ) {
        fclose(fp);
        fp = NULL;
        sys_err("create thread to end record failed...\n");
        g_thread_exit(NULL);
    }

    quit_flag = 0;
    sys_says("parse record: %d\n", quit_flag);

    while ( quit_flag != 1 ) {
        uint8_t buf[BUFSIZE];
        //sys_says("parse record: %d\n", quit_flag);

        /* Record some data ... */
        if (pa_simple_read(ps, buf, sizeof(buf), &error) < 0) {
            fprintf(stderr, __FILE__": pa_simple_read() failed: %s\n",
                    pa_strerror(error));
            goto finish;
        }

        /* And write it to STDOUT */
        if (loop_write(fp, buf, sizeof(buf)) != sizeof(buf)) {
            fprintf(stderr, __FILE__": write() failed: %s\n", strerror(errno));
            goto finish;
        }

        //fprintf(stdout, "cur db : %0.3lf\n", cur_db);
        //emit_sig_CurrentVolume();
    }

    fclose(fp);
    sys_says("end record: %d\n", quit_flag);

    //system("ls -l tmp.wav");
    //sys_says("\n");
    fp = NULL;
    emit_speech_signal("RecordEnd", NULL);
    g_thread_join(quit_thrd);

finish:

    if (ps) {
        pa_simple_free(ps);
    }

    g_thread_exit(NULL);
}

/* A simple routine calling UNIX write() in a loop */
static ssize_t loop_write(FILE *fp, const void *data, size_t size)
{
    ssize_t ret = 0;

    while (size > 0) {
        ssize_t r;

        if ((r = fwrite(data, 1, size, fp)) < 0) {
            return r;
        }

        if (r == 0) {
            break;
        }

        ret += r;
        data = (const uint8_t *) data + r;
        size -= (size_t) r;
    }

    return ret;
}

void record_pulse()
{
    int ret;
    GThread *recd_thrd = NULL;

    emit_speech_signal ("RecordStart", NULL);
    sys_says("begin record\n");
    recd_thrd = g_thread_new("pulse record", (GThreadFunc)parse_record, NULL);

    if ( recd_thrd == NULL ) {
        sys_err("create thread to parse record failed...\n");
        return ;
    }

    g_thread_join(recd_thrd);

    return ;
}

void emit_speech_signal(const char *signame, const double *sigvalue )
{
    DBusConnection *dbus_conn = NULL;
    DBusError dbus_err;
    DBusMessage *msg = NULL;
    DBusMessageIter args;

    dbus_error_init(&dbus_err);
    dbus_conn = dbus_bus_get(DBUS_BUS_SESSION, &dbus_err);

    if ( dbus_error_is_set(&dbus_err) ) {
        sys_err("get conn error : %s\n", dbus_err.message);
        dbus_error_free(&dbus_err);
    }

    if ( dbus_conn == NULL ) {
        sys_err("conn NULL...\n");
        return ;
    }

    msg = dbus_message_new_signal (DBUS_SPEECH_PATH, DBUS_SPEECH_INFACE, 
            signame);

    if ( msg == NULL ) {
        sys_err("msg NULL...\n");
        return ;
    }

    //append args onto signal
    if ( sigvalue ) {
        dbus_message_iter_init_append(msg, &args);

        if ( !dbus_message_iter_append_basic(&args, 
                    DBUS_TYPE_DOUBLE, sigvalue) ) {
            sys_says("Out Of Memory in append args\n");
            return ;
        }
    }

    //send message
    if ( !dbus_connection_send(dbus_conn, msg, NULL) ) {
        sys_err("Out Of Memory!\n");
        return ;
    }

    dbus_connection_flush(dbus_conn);
    //sys_says("%s Signal has been sent...\n", signame);

    dbus_message_unref(msg);
}

void init_pulse()
{
    pa_glib_mainloop *pa_ml = NULL;
    pa_mainloop_api *pa_mlapi = NULL;
    pa_context *pa_ctx = NULL;

    pa_ml = pa_glib_mainloop_new(g_main_context_default());
    pa_mlapi = pa_glib_mainloop_get_api(pa_ml);
    pa_ctx = pa_context_new(pa_mlapi, "deepin");

    pa_context_connect(pa_ctx, NULL, (pa_context_flags_t)0, NULL);
    pa_context_set_state_callback(pa_ctx, pa_state_cb, NULL);
}

void pa_source_cb(pa_context *c, const pa_source_info *i, int eol, void *userdata)
{
    if (eol > 0) {
        return;
    }

    int n;
    pa_cvolume source_volume;
    pa_operation *pa_op = NULL;
    source_volume.channels = i->volume.channels;

    // 设置每个声音的值
    for (n = 0; n < i->volume.channels; n++) {
        //source_volume.values[n] = i->volume.values[n];
        source_volume.values[n] = 10000;    // 15%
    }

    if (!(pa_op = pa_context_set_source_volume_by_name(c,
                  i->name, &source_volume, NULL, NULL))) {
        printf("pa_context_set_sink_volume_by_name() failed");
        return;
    }

    pa_operation_unref(pa_op);
    printf("set volume...\n");
}

void m_pa_server_info_cb(pa_context *c, const pa_server_info *i, void *userdate)
{
    // pa_server_info 结构体： http://freedesktop.org/software/pulseaudio/doxygen/structpa__server__info.html
    // 当时正在使用的输入设备
    //printf("default source; %s\n", i->default_source_name);
    pa_operation *pa_op = NULL;

    if (!(pa_op =  pa_context_get_source_info_by_name(c, i->default_source_name, pa_source_cb, NULL))) {
        printf("pa_context_get_source_info_list() failed");
        exit(-1);
    }

    pa_operation_unref(pa_op);
}


void pa_state_cb(pa_context *c, void *userdata)
{
    pa_context_state_t state;

    state = pa_context_get_state(c);

    switch  (state) {
            // There are just here for reference
        case PA_CONTEXT_UNCONNECTED:
        case PA_CONTEXT_CONNECTING:
        case PA_CONTEXT_AUTHORIZING:
        case PA_CONTEXT_SETTING_NAME:
            break;
        case PA_CONTEXT_FAILED:
        case PA_CONTEXT_TERMINATED:
            exit(-1);
            break;
        case PA_CONTEXT_READY: {
            pa_operation *pa_op = NULL;

            if (!(pa_op = pa_context_get_server_info(c, m_pa_server_info_cb, NULL))) {
                printf("pa_context_get_server_info() failed");
                return;
            }

            pa_operation_unref(pa_op);

            if (m_pa_stream_connect(c) < 0) {
                exit(-1);
            }

            break;
        }
    }
}

void on_monitor_read_callback(pa_stream *p, size_t length, void *userdata)
{
    const void *data;
    double v;

    /*printf("read callback length: %d\n", length);*/
    /*printf("\tget_device_index: %d\n", pa_stream_get_device_index(p));*/
    /*printf("\tget_device_name: %s\n", pa_stream_get_device_name(p));*/
    /*printf("\tget_monitor_stream: %d\n", pa_stream_get_monitor_stream(p));*/
    if (pa_stream_peek(p, &data, &length) < 0) {
        printf("Failed to read data from stream\n");
        return;
    }

    assert(length > 0);
    assert(length % sizeof(float) == 0);

    v = ((const float *) data)[length / sizeof(float) - 1];

    pa_stream_drop(p);

    if (v < 0) {
        v = 0;
    }

    if (v > 1) {
        v = 1;
    }

    //printf("\tread callback peek: %f\n", v);
    cur_db = v;

    if ( first_db < 0.00001 ) {
        first_db = cur_db;
        sys_says("first db : %lf\n\n", first_db);
    }/* else if (first_db > cur_db) {

        first_db = cur_db;
        sys_says("first db : %lf\n\n", first_db);
    }*/
    //printf("%lf\n", cur_db);
    emit_speech_signal("CurrentVolume", &cur_db);
}

void on_monitor_suspended_callback(pa_stream *p, void *userdata)
{
    if (pa_stream_is_suspended(p)) {
        printf("suspend callback\n");
    }
}

//static int m_pa_stream_connect(pa_context *pa_ctx)
int m_pa_stream_connect(pa_context *pa_ctx)
{
    if (pa_context_get_server_protocol_version (pa_ctx) < 13) {
        return -1;
    }

    //printf("server version: %d\n", pa_context_get_server_protocol_version(pa_ctx));
    if (s) {
        pa_stream_disconnect(s);
        pa_stream_unref(s);
    }

    pa_proplist  *proplist;

    pa_buffer_attr attr;
    pa_sample_spec ss;

    int res;
    //char dev_name[40];

    // pa_sample_spec
    ss.channels = 1;
    ss.format = PA_SAMPLE_FLOAT32;
    ss.rate = 25;

    // pa_buffer_attr
    memset(&attr, 0, sizeof(attr));
    attr.fragsize = sizeof(float);
    attr.maxlength = (uint32_t) - 1;

    // pa_proplist
    proplist = pa_proplist_new ();
    pa_proplist_sets (proplist, PA_PROP_APPLICATION_ID, "Deepin Sound Settings");

    // create new stream
    if (!(s = pa_stream_new_with_proplist(pa_ctx, "Deepin Sound Settings", &ss, NULL, proplist))) {
        fprintf(stderr, "pa_stream_new error\n");
        return -2;
    }

    pa_proplist_free(proplist);

    pa_stream_set_read_callback(s, on_monitor_read_callback, NULL);
    pa_stream_set_suspended_callback(s, on_monitor_suspended_callback, NULL);

    res = pa_stream_connect_record(s, NULL, &attr,
                                   (pa_stream_flags_t) (PA_STREAM_DONT_MOVE
                                           | PA_STREAM_PEAK_DETECT
                                           | PA_STREAM_ADJUST_LATENCY));

    if (res < 0) {
        fprintf(stderr, "Failed to connect monitoring stream\n");
        return -3;
    }

    return 0;
}
