/**
 * Copyright (c) 2011 ~ 2012 Deepin, Inc.
 *               2011 ~ 2012 jouyouyun
 *
 * Author:      jouyouyun <jouyouwen717@gmail.com>
 * Maintainer:  jouyouyun <jouyouwen717@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 **/

#include "dbus_speech.h"
#include "msp_custom.h"
#include "qisr.h"

void FinalizeQISRSession (const char* sess_id, const char* reason);

#define BUFFER_LEN (1024 * 4)

int
InitQisr ()
{
    int ret = 0;
    const char config[] = "appid=51777175";

    ret = QISRInit (config);
    if ( ret != MSP_SUCCESS ) {
        printf ("QISRInit: init qisr failed!\n"
                "\tError Code: %d\n", ret);
        return -1;
    }

    return 0;
}

void
FinalizeQisr ()
{
    int ret = 0;

    ret = QISRFini ();
    if ( ret != MSP_SUCCESS ) {
        printf ("QISRFini: finalize qisr failed!\n"
                "\tError Code: %d\n", ret);
        return ;
    }

    return ;
}

void
FinalizeQISRSession (const char* sess_id, const char* reason)
{
    int ret = 0;

    if ( sess_id == NULL ) {
        printf ("params is null in finalize session...\n");
        return ;
    }

    ret = QISRSessionEnd (sess_id, reason);
    if ( ret != MSP_SUCCESS ) {
        printf ("QISRSessionEnd: end session failed!\n"
                "\tError Code: %d\n", ret);
        return ;
    }

    return ;
}

gchar*
dbus_recog (const char* pcm_file)
{
    gchar* recog_ret = NULL;
    const char* sess_id = NULL;
    char buffer[BUFFER_LEN];
   /*
    * rse : 指定编码，默认为gb2312，可指定为utf8 | unicode
    * rst : 指定转写格式
    *       plain：直接转写
    *       json ：转写为json，编码只能为utf8
    */
    const char params[] = "sub=iat,ssm=1,auf=audio/L16;rate=16000,aue=speech,ent=sms16k,rse=utf8,rst=plain";
    int err_flag = 0;
    int ret = MSP_SUCCESS;
    int status = MSP_AUDIO_SAMPLE_CONTINUE;
    int ep_status = -1, rec_status = -1, rslt_status = -1;
    unsigned int len;
    FILE* fp = NULL;


    sess_id = QISRSessionBegin (NULL, params, &ret);
    if ( ret != MSP_SUCCESS ) {
        printf ("QISRSessionBegin: begin session failed!\n"
                "\tError Code: %d\n", ret);
        return NULL;
    }

    fp = fopen (pcm_file, "rb");
    if ( fp == NULL ) {
        printf ("open file %s error!\n", pcm_file);
        FinalizeQISRSession (sess_id, "OpenFileError");
        return NULL;
    }

    while ( !feof (fp) ) {
        len = (unsigned int)fread (buffer, 1, BUFFER_LEN, fp);
        //最后一块音频要使用last
        feof (fp) ? status = MSP_AUDIO_SAMPLE_LAST : 
            status = MSP_AUDIO_SAMPLE_CONTINUE;

        ret = QISRAudioWrite (sess_id, buffer, len, status, &ep_status, 
                &rec_status);
        if ( ret != MSP_SUCCESS ) {
            err_flag = 1;
            printf ("QISRAudioWrite: write audio failed!\n"
                    "\tError Code: %d\n", ret);
            break;
        }

        //检测音频后端点
        if ( ep_status == MSP_EP_AFTER_SPEECH ) {
            printf ("ep_status == MSP_EP_AFTER_SPEECH!\n");
            break;
        }

        //服务器返回结果
        if ( rec_status == MSP_REC_STATUS_SUCCESS ) {
            const char* result = QISRGetResult (sess_id, &rslt_status, 
                    0, &ret);
            if ( ret != MSP_SUCCESS ) {
                printf ("QISRGetResult: get return result failed!\n"
                        "\tError Code: %d\n", ret);
                err_flag = 1;
                break;
            }

            if ( rslt_status == MSP_REC_STATUS_NO_MATCH ) {
                printf ("get result no match!\n");
            } else {
                if ( result != NULL ) {
                    if ( recog_ret == NULL ) {
                        recog_ret = g_strdup (result);
                    } else {
                        gchar* tmp = g_strdup (recog_ret);
                        g_free (recog_ret);
                        recog_ret = g_strdup_printf ("%s%s", 
                                tmp, result);
                        g_free (tmp);
                    }
                    printf ("Result: %s\n", result);
                }
            }
        }

        //为了避免数据发送太快造成缓冲区溢出，这里暂停200ms
        usleep (50 * 1000);
    }
    fclose (fp);

    if ( !err_flag ) {
        //网络不好，数据未取完，可对循环次数作限定
        //int loop_count = 0;
        do {
            const char* result = QISRGetResult (sess_id, &rslt_status, 
                    0, &ret);
            if ( ret != MSP_SUCCESS ) {
                printf ("QISRGetResult: get return result failed!\n"
                        "\tError Code: %d\n", ret);
                err_flag = 1;
                break;
            }

            if ( rslt_status == MSP_REC_STATUS_NO_MATCH ) {
                printf ("get result no match!\n");
            } else {
                if ( result != NULL ) {
                    if ( recog_ret == NULL ) {
                        recog_ret = g_strdup (result);
                    } else {
                        gchar* tmp = g_strdup (recog_ret);
                        g_free (recog_ret);
                        recog_ret = g_strdup_printf ("%s%s", 
                                tmp, result);
                        g_free (tmp);
                    }
                    printf ("Result: %s\n", result);
                }
            }

            usleep (50 * 1000);
        } while (rslt_status != MSP_REC_STATUS_COMPLETE);
        //} while (rslt_status != MSP_REC_STATUS_COMPLETE && loop_count++ < 30);
    }

    FinalizeQISRSession (sess_id, "Normal");

    if ( err_flag ) {
        return NULL;
    }

    //g_print ("recog result: %s\n", recog_ret);
    return recog_ret;
}
