/**
 * Copyright (c) 2011 ~ 2012 Deepin, Inc.
 *               2011 ~ 2012 jouyouyun
 *
 * Author:      jouyouyun <jouyouwen717@gmail.com>
 * Maintainer:  jouyouyun <jouyouwen717@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 **/

#include "dbus_speech.h"
#include "msp_custom.h"
#include "qtts.h"

void FinalizeQTTSSession(const char* sess_id, const char* reason);

//wave 音频头格式
struct wave_pcm_hdr {
    char        riff[4];   // = "RIFF"
    int         size_8;     // = filesize - 8
    char        wave[4];   // = "WAVE"
    char        fmt[4];    // = "fmt"
    int         dwFmtSize;  // 下一结构体大小 = 16

    short int   format_tag;   // = PCM : 1
    short int   channels;     // = 1
    int         samples_per_sec;    // = rate : 8000 | 16000
    // 每秒字节数 = dwSamplesPerSec * wBitsPerSample / 8
    int         avg_bytes_per_sec;
    short int   block_align;    // 每采样点字节数：wBitsPerSample / 8
    short int   bits_per_sample;    // 量化比特数：8 | 16
    char        data[4];    // = "data"
    int         data_size;  // 纯数据长度: filesize - 44
};

//默认音频头数据
struct wave_pcm_hdr default_pcmwavhdr = 
{
    { 'R', 'I', 'F', 'F' },
    0,
    {'W', 'A', 'V', 'E'},
    {'f', 'm', 't', ' '},
    16,
    1,
    1,
    16000,
    32000,
    2,
    16,
    {'d', 'a', 't', 'a'},
    0  
};

int
dbus_synth (const char *str, const char *dest_path)
{
    struct wave_pcm_hdr pcm_wav_hdr = default_pcmwavhdr;
    char* audio_data = NULL;
    const char* sess_id = NULL;
    int ret = 0;
    int synth_status = MSP_TTS_FLAG_STILL_HAVE_DATA;
    unsigned int text_len = 0;
    unsigned int audio_len = 0;
    FILE* fp = NULL;

    int pos = 0;    //标记上一次合成到的位置
    int loop_count = 0; //标记去了几次结果
    int upload_flow = 0;    //上传流量
    int download_flow = 0;  //下载流量
    char param_value[32] = "";
    unsigned int value_len = 32;
    // vcn : 选择声音类型
    //  xiaoyan : woman
    //  xiaoyu  : man
    const char params[] = "ssm=1,auf=audio/L16;rate=16000,tte=UTF8";

    printf ( "Start to synth...\n" );
    if ( str == NULL || dest_path == NULL ) {
        printf ("params is null\n");
        return -1;
    }

    text_len = (unsigned int)strlen (str);
    sess_id = QTTSSessionBegin (params, &ret);
    if ( ret != MSP_SUCCESS ) {
        printf ("QTTSSessionBegin: qtts begin session failed!\n"
                "\tError Code: %d\n", ret);
        return ret;
    }

    ret = QTTSTextPut (sess_id, str, text_len, NULL);
    if ( ret != MSP_SUCCESS ) {
        printf ("QTTSTextPut: qtts put text failed!\n"
                "\tError Code: %d\n", ret);
        FinalizeQTTSSession (sess_id, "TextPutError");
        return ret;
    }

    fp = fopen (dest_path, "wb");
    if ( fp == NULL ) {
        printf ("Open file %s error!\n", dest_path);
        FinalizeQTTSSession (sess_id, "OpenFileError");
        return -1;
    }

    fwrite (&pcm_wav_hdr, 1, sizeof(pcm_wav_hdr), fp);
    while (TRUE) {
        audio_data = (char*)QTTSAudioGet (sess_id, &audio_len, 
                &synth_status, &ret);
        if ( ret != MSP_SUCCESS ) {
            printf ("QTTSAudioGet: qtts get audio failed!\n"
                    "\tError Code: %d\n", ret);
            break;
        }

        fwrite (audio_data, 1, audio_len, fp);
        pcm_wav_hdr.data_size += audio_len; //更新pcm数据的大小
        if ( synth_status == MSP_TTS_FLAG_DATA_END ) {
            printf ("QTTSAudioGet: get end of data!\n");
            break;
        }
    }

    //更新pcm文件头数据的大小
    pcm_wav_hdr.size_8 += pcm_wav_hdr.data_size + 36;
    //将更新的数据协会文件头部
    fseek (fp, 4, 0);
    fwrite (&pcm_wav_hdr.size_8, sizeof (pcm_wav_hdr.size_8), 
            1, fp);
    fseek (fp, 40, 0);
    fwrite (&pcm_wav_hdr.data_size, sizeof (pcm_wav_hdr.data_size), 
            1, fp);
    fclose (fp);

    FinalizeQTTSSession (sess_id, "Normal");
    return 0;
}

void
FinalizeQTTSSession(const char* sess_id, const char* reason)
{
    int ret = 0;

    if ( sess_id == NULL || reason == NULL ) {
        printf ("params is null in finalize_synth...\n");
        return ;
    }

    ret = QTTSSessionEnd (sess_id, reason);
    if ( ret != MSP_SUCCESS ) {
        printf ("QTTSSessionEnd: qtts session end failed!\n"
                "\tError Code: %d\n", ret);
        return ;
    }

    return ;
}

void
FinalizeSynth()
{
    int ret = 0;
    //
    //关闭引擎
    ret = QTTSFini();
    if ( ret != MSP_SUCCESS ) {
        printf ("QTTSFini: finalize failed!\n"
                "\tError Code: %d\n", ret);
        return ;
    }

    return ;
}

int
InitSynth ()
{
    int ret = 0;
    const char config[] = "appid=51777175";

    ret = QTTSInit (config);
    if ( ret != MSP_SUCCESS ) {
        printf ("QTTSInit: init failed!\n"
                "\tError Code: %d\n", ret);
        return -1;
    }

    return 0;
}
