#include <stdio.h>
#include <string.h>
#include <sqlite3.h>

#define ZH_LEN 4
#define EN_LEN 7
#define BUF_LEN 100

#define DATABASE_PATH "test.db"

#define sys_err(arg ...) { fprintf(stderr, arg); }
#define sys_says(arg ...) { fprintf(stdout, arg); }

/* 创建表格 */
void sql_create_table(const char *table_name )
{
	char sql[1024];
	sqlite3 *db = NULL;
	char *err_msg = NULL;
	
	if ( table_name == NULL ) {
		sys_err("insert table arguments error : table == NULL...\n");
		return ;
	}
	
	memset(sql, 0, 1024);
	sprintf(sql, "create table %s ( speech text primary key not null, cmd text, type text)", table_name );
	sys_says("\nsql : %s\n\n", sql);

	/* 打开内存数据库 */
	if ( SQLITE_OK != sqlite3_open(DATABASE_PATH, &db) ) {
		sys_err("search_index_table sqlite3_open failed...\n");
		return ;
	}
	
	if ( SQLITE_OK != sqlite3_exec( db, sql, 0, 0, &err_msg) ) {
		sys_err("create_table sqlite3_exec failed...\n");
		return ;
	}
	
	if ( SQLITE_OK != sqlite3_close(db) ) {
		sys_err("create_table sqlite3_close failed...\n");
		return ;
	}
	
	return ;
}

/* 插入记录 */
void sql_insert_table(const char *table_name, 
					const char *arg1, 
					const char *arg2, 
					const char *arg3 )
{
	char sql[1024];
	sqlite3 *db = NULL;
	char *err_msg = NULL;
	
	if ( table_name == NULL ) {
		sys_err("insert table arguments error : table == NULL...\n");
		return ;
	}
	
	memset(sql, 0, 1024);
	if ( strncmp("SpeechIndex", table_name, strlen(table_name)) == 0 ) {
		sprintf(sql, "insert into %s (speech, ProcessName, TableName) values(\'%s\', \'%s\', \'%s\')", table_name, arg1, arg2, arg3 );
	} else {
		sprintf(sql, "insert into %s (speech, cmd, type) values (\'%s\', \'%s\', \'%s\')", table_name, arg1, arg2, arg3 );
	}
	sys_says("\nsql : %s\n\n", sql);

	/* 打开内存数据库 */
	if ( SQLITE_OK != sqlite3_open(DATABASE_PATH, &db) ) {
		sys_err("search_index_table sqlite3_open failed...\n");
		return ;
	}
	
	if ( SQLITE_OK != sqlite3_exec( db, sql, 0, 0, &err_msg) ) {
		sys_err("insert table sqlite3_exec failed...\n");
		return ;
	}
	
	if ( SQLITE_OK != sqlite3_close(db) ) {
		sys_err("insert table sqlite3_close failed...\n");
		return ;
	}
	
	return ;
}

void insert_pinyin(const char *file_path)
{
	char sql[100];
	char zh[ZH_LEN];
	char en[EN_LEN];
	FILE *fp = NULL;
	sqlite3 *db = NULL;
	char *err_msg = NULL;
	
	if ( file_path == NULL ) {
		sys_says("insert pinyin argument error...\n");
		return ;
	}

	if ( (fp = fopen(file_path, "r")) == NULL ) {
		sys_says("open file failed...\n");
		return ;
	}
	
	if ( SQLITE_OK != sqlite3_open(DATABASE_PATH, &db) ) {
		sys_says("search_table sqlite3_open failed...\n");
		return ;
	}
	
	while ( !feof(fp) ) {
		memset(zh, 0, ZH_LEN);
		memset(en, 0, EN_LEN);
		memset(sql, 0, 100);
		
		fscanf(fp, "%s %s", zh, en);
		//sys_says("zh : %s\ten : %s\n", zh, en);
		
		if ( zh[0] == '\0' ) {
			continue;
		}
		sprintf(sql, "insert into zhIndex(zhcn, pinyin) \
				values(\'%s\', \'%s\')", zh, en);
		if ( SQLITE_OK != sqlite3_exec( db, sql, 0, 0, &err_msg ) ) {
			sys_says("zh : %s\ten : %s\n", zh, en);
			sys_says("sqlite3_exec failed...\n");
			//break;				
		}
	}
	fclose(fp);
	if ( SQLITE_OK != sqlite3_close(db) ) {
		sys_says("sqlite3_close failed...\n");
		return ;
	}
	
	return ;
}

int main(int argc, char **argv)
{
	//sql_create_table("geany");
	insert_pinyin(argv[1]);
	//sql_insert_table("geany", "geany", "geany", "cmd");
	//sql_insert_table("SpeechIndex", "geany", "geany", "cmd");
	
	return 0;
}
